@@ -8,7 +8,7 @@ use POPFile::Module;
 #
 # WordMangle.pm --- Mangle words for better classification
 #
-# Copyright (c) 2001-2006 John Graham-Cumming
+# Copyright (c) 2001-2009 John Graham-Cumming
 #
 #   This file is part of POPFile
 #
@@ -176,7 +176,7 @@ sub add_stopword
     # In Japanese mode, reject non EUC Japanese characters.
 
     if ( $lang eq 'Nihongo') {
-        if ( $stopword !~ /$euc_jp/i ) {
+        if ( $stopword !~ /^($euc_jp)+$/o ) {
             return 0;
         }
     } else {
@@ -204,7 +204,7 @@ sub remove_stopword
     # In Japanese mode, reject non EUC Japanese characters.
 
     if ( $lang eq 'Nihongo') {
-        if ( $stopword !~ /$euc_jp/i ) {
+        if ( $stopword !~ /^($euc_jp)+$/o ) {
             return 0;
         }
     } else {
