@@ -11,7 +11,7 @@ use POPFile::Module;
 # register specific parameters with this module.  This module also handles
 # POPFile's command line parsing
 #
-# Copyright (c) 2001-2006 John Graham-Cumming
+# Copyright (c) 2001-2009 John Graham-Cumming
 #
 #   This file is part of POPFile
 #
@@ -57,10 +57,6 @@ sub new
 
     $self->{pid_file__} = '';
 
-    # The time to delay checking of the PID file
-
-    $self->{pid_delay__} = 5;
-
     # The last time the PID was checked
 
     $self->{pid_check__} = time;
@@ -103,6 +99,11 @@ sub initialize
 
     $self->config_( 'piddir', './' );
 
+    # The default interval of checking pid file in seconds
+    # To turn off checking, set this option to 0
+
+    $self->config_( 'pidcheck_interval', 5 );
+
     # The default timeout in seconds for POP3 commands
 
     $self->global_config_( 'timeout', 60 );
@@ -116,6 +117,14 @@ sub initialize
 
     $self->global_config_( 'message_cutoff', 100000 );
 
+    # Checking for updates if off by default
+
+    $self->global_config_( 'update_check', 0 );
+
+    # The last time we checked for an update using the local epoch
+
+    $self->global_config_( 'last_update_check', 0 );
+
     # Register for the TICKD message which is sent hourly by the
     # Logger module.   We use this to hourly save the configuration file
     # so that POPFile's configuration is saved in case of a hard crash.
@@ -144,10 +153,10 @@ sub start
 
     $self->{started__} = 1;
 
-    # Check to see if the PID file is present, if it is then another POPFile
-    # may be running, warn the user and terminate, note the 0 at the end
-    # means that we allow the piddir to be absolute and outside the user
-    # sandbox
+    # Check to see if the PID file is present, if it is then another
+    # POPFile may be running, warn the user and terminate, note the 0
+    # at the end means that we allow the piddir to be absolute and
+    # outside the user sandbox
 
     $self->{pid_file__} = $self->get_user_path( $self->config_( 'piddir' ) . 'popfile.pid', 0 );
 
@@ -164,11 +173,14 @@ sub start
 #
 # service
 #
-# service() is a called periodically to give the module a chance to do housekeeping work.
+# service() is a called periodically to give the module a chance to do
+# housekeeping work.
+#
+# If any problem occurs that requires POPFile to shutdown service()
+# should return 0 and the top level process will gracefully terminate
+# POPFile including calling all stop() methods.  In normal operation
+# return 1.
 #
-# If any problem occurs that requires POPFile to shutdown service() should return 0 and
-# the top level process will gracefully terminate POPFile including calling all stop()
-# methods.  In normal operation return 1.#
 # ----------------------------------------------------------------------------
 sub service
 {
@@ -176,13 +188,15 @@ sub service
 
     my $time = time;
 
-    if ( $self->{pid_check__} <= ( $time - $self->{pid_delay__})) {
+    if ( $self->config_( 'pidcheck_interval' ) > 0 ) {
+        if ( $self->{pid_check__} <= ( $time - $self->config_( 'pidcheck_interval' ))) {
 
-        $self->{pid_check__} = $time;
+            $self->{pid_check__} = $time;
 
-        if ( !$self->check_pid_() ) {
-            $self->write_pid_();
-            $self->log_( 0, "New POPFile instance detected and signalled" );
+            if ( !$self->check_pid_() ) {
+                $self->write_pid_();
+                $self->log_( 0, "New POPFile instance detected and signalled" );
+            }
         }
     }
 
@@ -236,18 +250,19 @@ sub live_check_
     if ( $self->check_pid_() ) {
 
         my $oldpid = $self->get_pid_();
+        my $wait_time = $self->config_( 'pidcheck_interval' ) * 2;
 
-        my $error = "\n\nA copy of POPFile appears to be running.\n Attempting to signal the previous copy.\n Waiting " . ($self->{pid_delay__} * 2) . " seconds for a reply.\n";
+        my $error = "\n\nA copy of POPFile appears to be running.\n Attempting to signal the previous copy.\n Waiting $wait_time seconds for a reply.\n";
 
         $self->delete_pid_();
 
         print STDERR $error;
 
-        select(undef, undef, undef, ($self->{pid_delay__} * 2));
+        select( undef, undef, undef, $wait_time );
 
         my $pid = $self->get_pid_();
 
-        if (defined($pid)) {
+        if ( defined($pid) ) {
             $error = "\n A copy of POPFile is running.\n It has signaled that it is alive with process ID: $pid\n";
             print STDERR $error;
             return $pid;
@@ -276,7 +291,8 @@ sub check_pid_
 #
 # get_pid_
 #
-# returns the pidfile proccess ID if a pid file is present, undef otherwise (0 might be a valid PID)
+# returns the pidfile proccess ID if a pid file is present, undef
+# otherwise (0 might be a valid PID)
 #
 # ----------------------------------------------------------------------------
 sub get_pid_
@@ -349,8 +365,8 @@ sub parse_command_line
     #
     # So its possible to do
     #
-    # --set bayes_param=value --set=-bayes_param=value --set
-    # -bayes_param=value -- -bayes_param value
+    # --set bayes_param=value --set=-bayes_param=value
+    # --set -bayes_param=value -- -bayes_param value
 
     if ( !GetOptions( "set=s" => \@set_options ) ) {
         return 0;
@@ -436,8 +452,7 @@ sub upgrade_parameter__
                      # Parameters that are now handled by Classifier::Bayes
 
                      'corpus',                   'bayes_corpus',
-                     'unclassified_probability',
-                         'bayes_unclassified_probability',
+                     'unclassified_probability', 'bayes_unclassified_probability',
 
                      # Parameters that are now handled by
                      # POPFile::Configuration
@@ -477,7 +492,7 @@ sub upgrade_parameter__
                      'update_check',             'html_update_check',
                      'ui_port',                  'html_port',
 
-                     # Parameters the have moved from the UI::HTML to
+                     # Parameters that have moved from the UI::HTML to
                      # POPFile::History
 
                      'archive',                  'history_archive',
@@ -489,6 +504,12 @@ sub upgrade_parameter__
                      'html_archive_dir',         'history_archive_dir',
                      'html_history_days',        'history_history_days',
 
+                     # Parameters that have moved from UI::HTML to
+                     # global to POPFile
+
+                     'html_update_check',        'GLOBAL_update_check',
+                     'html_last_update_check',   'GLOBAL_last_update_check',
+
     ); # PROFILE BLOCK STOP
 
     if ( defined( $upgrades{$parameter} ) ) {
@@ -529,10 +550,10 @@ sub load_configuration
                 # so that the Japanese users can use insert.pl
                 # etc. which rely on knowing the language
 
-                if (defined($self->{configuration_parameters__}{$parameter}) ||
-                    ( $parameter eq 'html_language' ) ) {
-                    $self->{configuration_parameters__}{$parameter}{value} =
-                        $value;
+                if (defined($self->{configuration_parameters__}{$parameter}) ||  # PROFILE BLOCK START
+                    ( $parameter eq 'html_language' ) ) {                        # PROFILE BLOCK STOP
+                    $self->{configuration_parameters__}{$parameter}{value} =   # PROFILE BLOCK START
+                        $value;                                                # PROFILE BLOCK STOP
                 } else {
                     $self->{deprecated_parameters__}{$parameter} = $value;
                 }
@@ -561,7 +582,7 @@ sub save_configuration
         return;
     }
 
-    if ( open CONFIG, '>' . $self->get_user_path( 'popfile.cfg' ) ) {
+    if ( open CONFIG, '>' . $self->get_user_path( 'popfile.cfg.tmp' ) ) {
         $self->{save_needed__} = 0;
 
         foreach my $key (sort keys %{$self->{configuration_parameters__}}) {
@@ -569,6 +590,9 @@ sub save_configuration
         }
 
         close CONFIG;
+
+        rename $self->get_user_path( 'popfile.cfg.tmp' ), # PROFILE BLOCK START
+               $self->get_user_path( 'popfile.cfg' );     # PROFILE BLOCK STOP
     }
 }
 
@@ -615,9 +639,9 @@ sub path_join__
 
     $sandbox = 1 if ( !defined( $sandbox ) );
 
-    if ( ( $right =~ /^\// ) ||
+    if ( ( $right =~ /^\// ) ||               # PROFILE BLOCK START
          ( $right =~ /^[A-Za-z]:[\/\\]/ ) ||
-         ( $right =~ /\\\\/ ) ) {
+         ( $right =~ /\\\\/ ) ) {             # PROFILE BLOCK STOP
         if ( $sandbox ) {
             $self->log_( 0, "Attempt to access path $right outside sandbox" );
             return undef;
@@ -685,8 +709,8 @@ sub is_default
 {
     my ( $self, $name ) = @_;
 
-    return ( $self->{configuration_parameters__}{$name}{value} eq
-             $self->{configuration_parameters__}{$name}{default} );
+    return ( $self->{configuration_parameters__}{$name}{value} eq   # PROFILE BLOCK START
+             $self->{configuration_parameters__}{$name}{default} ); # PROFILE BLOCK STOP
 }
 
 # GETTERS
@@ -706,4 +730,3 @@ sub deprecated_parameter
 }
 
 1;
-
