@@ -9,7 +9,7 @@ use Digest::MD5;
 #
 # This module handles proxying the POP3 protocol for POPFile.
 #
-# Copyright (c) 2001-2006 John Graham-Cumming
+# Copyright (c) 2001-2009 John Graham-Cumming
 #
 #   This file is part of POPFile
 #
@@ -82,7 +82,7 @@ sub initialize
     my ( $self ) = @_;
 
     # Enabled by default
-    $self->config_( 'enabled', 1);
+    $self->config_( 'enabled', 1 );
 
     # By default we don't fork on Windows
     $self->config_( 'force_fork', ($^O eq 'MSWin32')?0:1 );
@@ -104,8 +104,8 @@ sub initialize
     $self->config_( 'separator', ':' );
 
     # The welcome string from the proxy is configurable
-    $self->config_( 'welcome_string',
-        "POP3 POPFile ($self->{version_}) server ready" );
+    $self->config_( 'welcome_string',                      # PROFILE BLOCK START
+        "POP3 POPFile ($self->{version_}) server ready" ); # PROFILE BLOCK STOP
 
     return $self->SUPER::initialize();
 }
@@ -143,8 +143,10 @@ sub start
                                          'pop3-chain-panel.thtml',
                                          $self );                      # PROFILE BLOCK STOP
 
-    if ( $self->config_( 'welcome_string' ) =~ /^POP3 POPFile \(v\d+\.\d+\.\d+\) server ready$/ ) { # PROFILE BLOCK START
-        $self->config_( 'welcome_string', "POP3 POPFile ($self->{version_}) server ready" );        # PROFILE BLOCK STOP
+    if ( $self->config_( 'welcome_string' ) =~                # PROFILE BLOCK START
+         /^POP3 POPFile \(v\d+\.\d+\.\d+\) server ready$/ ) { # PROFILE BLOCK STOP
+        $self->config_( 'welcome_string',                                  # PROFILE BLOCK START
+                        "POP3 POPFile ($self->{version_}) server ready" ); # PROFILE BLOCK STOP
     }
 
     return $self->SUPER::start();
@@ -154,7 +156,8 @@ sub start
 #
 # child__
 #
-# The worker method that is called when we get a good connection from a client
+# The worker method that is called when we get a good connection from
+# a client
 #
 # $client         - an open stream to a POP3 client
 # $session        - API session key
@@ -180,14 +183,15 @@ sub child__
     # Tell the client that we are ready for commands and identify our
     # version number
 
-    $self->tee_( $client, "+OK " . $self->config_( 'welcome_string' ) . "$eol" );
+    $self->tee_( $client, "+OK " . $self->config_( 'welcome_string' ) . # PROFILE BLOCK START
+               "$eol" );                                                # PROFILE BLOCK STOP
 
     # Compile some configurable regexp's once
 
     my $s = $self->config_( 'separator' );
     $s =~ s/(\$|\@|\[|\]|\(|\)|\||\?|\*|\.|\^|\+)/\\$1/;
 
-    my $transparent  = "^USER ([^$s])+\$";
+    my $transparent  = "^USER ([^$s]+)\$";
     my $user_command = "USER ([^$s]+)($s(\\d+))?$s([^$s]+)($s([^$s]+))?";
     my $apop_command = "APOP ([^$s]+)($s(\\d+))?$s([^$s]+) (.*?)";
 
@@ -227,15 +231,17 @@ sub child__
         # that the user can say host:username:ssl,apop if both are
         # needed
 
-        if ( $command =~ /$transparent/ ) {
+        if ( $command =~ /$transparent/i ) {
             if ( $self->config_( 'secure_server' ) ne '' )  {
-                if ( $mail = $self->verify_connected_( $mail, $client,  $self->config_( 'secure_server' ), $self->config_( 'secure_port' ) ) )  {
+                if ( $mail = $self->verify_connected_( $mail, $client,  # PROFILE BLOCK START
+                        $self->config_( 'secure_server' ),
+                        $self->config_( 'secure_port' ) ) )  {          # PROFILE BLOCK STOP
                     last if ($self->echo_response_($mail, $client, $command) == 2 );
                 } else {
                     next;
                 }
             } else {
-                $self->tee_(  $client, "-ERR Transparent proxying not configured: set secure server/port$eol" );
+                $self->tee_(  $client, "-ERR Transparent proxying not configured: set secure server/port ( command you sent: '$command' )$eol" );
             }
 
             next;
@@ -248,10 +254,21 @@ sub child__
                 $self->mq_post_( 'LOGIN', $user );
 
                 my $ssl = defined( $options ) && ( $options =~ /ssl/i );
+
+                # We cannot use the concurrent POP3 connections with SSL on
+                # Windows because one of SSL support modules (Net::SSLeay) is
+                # not thread-safe. ActivePerl for Windows emulates fork() by
+                # multiple threads.
+
+                if ( $ssl && ( $^O eq 'MSWin32' ) && $self->config_( 'force_fork' ) ) {
+                    $self->tee_( $client, "-ERR On Windows, SSL support cannot be used with concurrent POP3 connections$eol" );
+                    next;
+                }
+
                 $port = $ssl?995:110 if ( !defined( $port ) );
 
-                if ( $mail = $self->verify_connected_( $mail, $client,
-                                 $host, $port, $ssl ) )  {
+                if ( $mail = $self->verify_connected_( $mail, $client, # PROFILE BLOCK START
+                                 $host, $port, $ssl ) )  {             # PROFILE BLOCK STOP
 
                     if ( defined( $options ) && ( $options =~ /apop/i ) ) {
 
@@ -324,18 +341,18 @@ sub child__
                 my $md5hex = $md5->hexdigest;
                 $self->log_( 2, "digest='$md5hex'" );
 
-                my ($response, $ok) =
+                my ( $response, $ok ) =                              # PROFILE BLOCK START
                     $self->get_response_( $mail, $client,
-                        "APOP $self->{apop_user__} $md5hex", 0, 1 );
-                if ( ( $ok == 1 ) &&
-                     ( $response =~ /$self->{good_response_}/ ) ) {
+                        "APOP $self->{apop_user__} $md5hex", 0, 1 ); # PROFILE BLOCK STOP
+                if ( ( $ok == 1 ) &&                                # PROFILE BLOCK START
+                     ( $response =~ /$self->{good_response_}/ ) ) { # PROFILE BLOCK STOP
 
                     # authentication OK, toss the hello response and
                     # return password ok
 
                     $self->tee_( $client, "+OK password ok$eol" );
                 } else {
-                    $self->tee_( $client, "$response" );
+                    $self->tee_( $client, $response );
                 }
              } else {
                last if ($self->echo_response_($mail, $client, $command) == 2 );
@@ -344,12 +361,12 @@ sub child__
         }
 
         # User is issuing the APOP command to start a session with the
-        # remote server We'd need a copy of the plaintext password to
+        # remote server. We'd need a copy of the plaintext password to
         # support this.
 
         if ( $command =~ /$apop_command/io ) {
-            $self->tee_( $client,
-              "-ERR APOP not supported between mail client and POPFile.$eol" );
+            $self->tee_( $client,                                               # PROFILE BLOCK START
+              "-ERR APOP not supported between mail client and POPFile.$eol" ); # PROFILE BLOCK STOP
 
             # TODO: Consider implementing a host:port:username:secret
             # hash syntax for proxying the APOP command
@@ -359,9 +376,11 @@ sub child__
 
         # Secure authentication
 
-        if ( $command =~ /AUTH ([^ ]+)/ ) {
+        if ( $command =~ /AUTH ([^ ]+)/i ) {
             if ( $self->config_( 'secure_server' ) ne '' )  {
-                if ( $mail = $self->verify_connected_( $mail, $client,  $self->config_( 'secure_server' ), $self->config_( 'secure_port' ) ) )  {
+                if ( $mail = $self->verify_connected_( $mail, $client,  # PROFILE BLOCK START
+                        $self->config_( 'secure_server' ),
+                        $self->config_( 'secure_port' ) ) )  {          # PROFILE BLOCK STOP
 
                     # Loop until we get -ERR or +OK
 
@@ -383,9 +402,11 @@ sub child__
             next;
         }
 
-        if ( $command =~ /AUTH/ ) {
+        if ( $command =~ /AUTH/i ) {
             if ( $self->config_( 'secure_server' ) ne '' )  {
-                if ( $mail = $self->verify_connected_( $mail, $client,  $self->config_( 'secure_server' ), $self->config_( 'secure_port' ) ) )  {
+                if ( $mail = $self->verify_connected_( $mail, $client,  # PROFILE BLOCK START
+                        $self->config_( 'secure_server' ),
+                        $self->config_( 'secure_port' ) ) )  {          # PROFILE BLOCK STOP
                     my $response = $self->echo_response_($mail, $client, "AUTH" );
                     last if ( $response == 2 );
                     if ( $response == 0 ) {
@@ -465,45 +486,46 @@ sub child__
 
             if ( $2 ne '99999999' )  {
                 if ( $self->config_( 'toptoo' ) == 1 ) {
-                    my $response =
-                        $self->echo_response_( $mail, $client, "RETR $count" );
+                    my $response =                                              # PROFILE BLOCK START
+                        $self->echo_response_( $mail, $client, "RETR $count" ); # PROFILE BLOCK STOP
                     last if ( $response == 2 );
                     if ( $response == 0 ) {
 
                         # Classify without echoing to client, saving
                         # file for later RETR's
 
-                        my ( $class, $slot ) =
-                             $self->{classifier__}->classify_and_modify(
-                                 $session, $mail, $client, 0, '', 0, 0 );
+                        my ( $class, $slot ) =                           # PROFILE BLOCK START
+                            $self->{classifier__}->classify_and_modify(
+                                $session, $mail, $client, 0, '', 0, 0 ); # PROFILE BLOCK STOP
 
-                        $downloaded{$count} = $slot;
+                        $downloaded{$count}{slot}  = $slot;
+                        $downloaded{$count}{class} = $class;
 
                         # Note that the 1 here indicates that
                         # echo_response_ does not send the response to
                         # the client.  The +OK has already been sent
                         # by the RETR
 
-                        $response =
+                        $response =                                # PROFILE BLOCK START
                             $self->echo_response_( $mail, $client,
-                                $command, 1 );
+                                $command, 1 );                     # PROFILE BLOCK STOP
                         last if ( $response == 2 );
                         if ( $response == 0 ) {
 
                             # Classify with pre-defined class, without
                             # saving, echoing to client
 
-                            $self->{classifier__}->classify_and_modify(
-                                $session, $mail, $client, 1, $class, $slot, 1 );
+                            $self->{classifier__}->classify_and_modify(          # PROFILE BLOCK START
+                                $session, $mail, $client, 1, $class, $slot, 1 ); # PROFILE BLOCK STOP
                         }
                     }
                 } else {
-                    my $response =
-                        $self->echo_response_( $mail, $client, $command );
+                    my $response =                                         # PROFILE BLOCK START
+                        $self->echo_response_( $mail, $client, $command ); # PROFILE BLOCK STOP
                     last if ( $response == 2 );
                     if ( $response == 0 ) {
                         $self->echo_to_dot_( $mail, $client );
-		    }
+                    }
                 }
 
                 next;
@@ -517,12 +539,14 @@ sub child__
 
         if ( $command =~ /CAPA/i ) {
             if ( $mail || $self->config_( 'secure_server' ) ne '' )  {
-                if ( $mail || ( $mail = $self->verify_connected_( $mail, $client, $self->config_( 'secure_server' ), $self->config_( 'secure_port' ) ) ) )  {
+                if ( $mail || ( $mail = $self->verify_connected_( $mail, $client, # PROFILE BLOCK START
+                                   $self->config_( 'secure_server' ),
+                                   $self->config_( 'secure_port' ) ) ) )  {       # PROFILE BLOCK STOP
                     my $response = $self->echo_response_($mail, $client, "CAPA" );
                     last if ( $response == 2 );
                     if ( $response == 0 ) {
                         $self->echo_to_dot_( $mail, $client );
-		    }
+                    }
                 } else {
                     next;
                 }
@@ -564,9 +588,10 @@ sub child__
             my $class;
             my $file;
 
-            if ( defined($downloaded{$count}) &&
-                 ( $file = $self->{history__}->get_slot_file( $downloaded{$count} ) ) &&
-                 (open RETRFILE, "<$file") ) {
+            if ( defined($downloaded{$count}) &&             # PROFILE BLOCK START
+                 ( $file = $self->{history__}->get_slot_file(
+                        $downloaded{$count}{slot} ) ) &&
+                 ( open RETRFILE, "<$file" ) ) {             # PROFILE BLOCK STOP
 
                 # act like a network stream
 
@@ -580,28 +605,14 @@ sub child__
 
                 $self->tee_( $client, "+OK " . ( -s $file ) . " bytes from POPFile cache$eol" );
 
-                # Load the last classification
+                # echo file, inserting known classification,
+                # without saving
 
-                my ( $id, $from, $to, $cc, $subject,
-                    $date, $hash, $inserted, $bucket, $reclassified ) =
-                    $self->{history__}->get_slot_fields( $downloaded{$count} );
-
-                if ( $bucket ne 'unknown class' ) {
-
-                    # echo file, inserting known classification,
-                    # without saving
-
-                    ($class, undef) = $self->{classifier__}->classify_and_modify( $session, \*RETRFILE, $client, 1, $bucket, $downloaded{$count} );
-                    print $client ".$eol";
-
-                } else {
-
-                    # If the class wasn't saved properly, classify
-                    # from disk normally
-
-                    ($class, undef) = $self->{classifier__}->classify_and_modify( $session, \*RETRFILE, $client, 1, '', 0 );
-                    print $client ".$eol";
-                }
+                ( $class, undef ) = $self->{classifier__}->classify_and_modify(  # PROFILE BLOCK START
+                        $session, \*RETRFILE, $client, 1, 
+                        $downloaded{$count}{class},
+                        $downloaded{$count}{slot} );                             # PROFILE BLOCK STOP
+                print $client ".$eol";
 
                 close RETRFILE;
             } else {
@@ -618,12 +629,15 @@ sub child__
                 last if ( $response == 2 );
                 if ( $response == 0 ) {
                     my $slot;
-                    ( $class, $slot ) = $self->{classifier__}->classify_and_modify( $session, $mail, $client, 0, '', 0 );
+                    ( $class, $slot ) =                             # PROFILE BLOCK START
+                        $self->{classifier__}->classify_and_modify(
+                            $session, $mail, $client, 0, '', 0 );   # PROFILE BLOCK STOP
 
                     # Note locally that file has been retrieved if the
                     # full thing has been saved to disk
 
-                    $downloaded{$count} = $slot;
+                    $downloaded{$count}{slot}  = $slot;
+                    $downloaded{$count}{class} = $class;
                 }
             }
 
@@ -652,7 +666,7 @@ sub child__
             last if ( $self->echo_response_($mail, $client, $command ) == 2 );
             next;
         } else {
-            $self->tee_(  $client, "-ERR unknown command or bad syntax$eol" );
+            $self->tee_( $client, "-ERR unknown command or bad syntax$eol" );
             next;
         }
 
@@ -690,13 +704,13 @@ sub configure_item
     } else {
         if ( $name eq 'pop3_security' ) {
             $templ->param( 'POP3_Security_Local' => ( $self->config_( 'local' ) == 1 ) );
-	} else {
+        } else {
             if ( $name eq 'pop3_chain' ) {
                 $templ->param( 'POP3_Chain_Secure_Server' => $self->config_( 'secure_server' ) );
                 $templ->param( 'POP3_Chain_Secure_Port' => $self->config_( 'secure_port' ) );
-	    } else {
+            } else {
                 $self->SUPER::configure_item( $name, $templ, $language );
-	    }
+            }
         }
     }
 }
@@ -719,12 +733,16 @@ sub validate_item
 
     if ( $name eq 'pop3_configuration' ) {
         if ( defined($$form{pop3_port}) ) {
-            if ( ( $$form{pop3_port} >= 1 ) && ( $$form{pop3_port} < 65536 ) ) {
+            if ( ( $$form{pop3_port} >= 1 ) && ( $$form{pop3_port} < 65536 ) && ( $self->module_config_( 'html', 'port' ) ne $$form{pop3_port} ) ) {
                 $self->config_( 'port', $$form{pop3_port} );
                 $templ->param( 'POP3_Configuration_If_Port_Updated' => 1 );
                 $templ->param( 'POP3_Configuration_Port_Updated' => sprintf( $$language{Configuration_POP3Update}, $self->config_( 'port' ) ) );
             } else {
-                $templ->param( 'POP3_Configuration_If_Port_Error' => 1 );
+                if ( ( $self->module_config_( 'html', 'port' ) ne $$form{pop3_port} ) ) {
+                  $templ->param( 'POP3_Configuration_If_Port_Error' => 1 );
+                } else {
+                  $templ->param( 'POP3_Configuration_If_UI_Port_Error' => 1 );
+                }
             }
         }
 
@@ -756,7 +774,7 @@ sub validate_item
             $self->config_( 'secure_server', $$form{server} );
             $templ->param( 'POP3_Chain_If_Server_Updated' => 1 );
             $templ->param( 'POP3_Chain_Server_Updated' => sprintf( $$language{Security_SecureServerUpdate}, $self->config_( 'secure_server' ) ) );
-	}
+        }
 
         if ( defined($$form{sport}) ) {
             if ( ( $$form{sport} >= 1 ) && ( $$form{sport} < 65536 ) ) {
@@ -774,3 +792,4 @@ sub validate_item
     $self->SUPER::validate_item( $name, $templ, $language, $form );
 }
 
+1;
