@@ -4,7 +4,7 @@ package Proxy::Proxy;
 #
 # This module implements the base class for all POPFile proxy Modules
 #
-# Copyright (c) 2001-2006 John Graham-Cumming
+# Copyright (c) 2001-2009 John Graham-Cumming
 #
 #   This file is part of POPFile
 #
@@ -83,6 +83,8 @@ sub new
     #
     # $self->{good_response_}            = '';
 
+    $self->{ssl_not_supported_error_}  = '-ERR SSL connection is not supported since required modules are not installed';
+
     # Connect Banner returned by the real server
     $self->{connect_banner__} = '';
 
@@ -170,8 +172,9 @@ EOM
 #
 # stop
 #
-# Called when POPFile is closing down, this is the last method that will get called before
-# the object is destroyed.  There is no return value from stop().
+# Called when POPFile is closing down, this is the last method that
+# will get called before the object is destroyed.  There is no return
+# value from stop().
 #
 # ----------------------------------------------------------------------------
 sub stop
@@ -182,8 +185,9 @@ sub stop
         $self->{classifier__}->release_session_key( $self->{api_session__} );
     }
 
-    # Need to close all the duplicated file handles, this include the POP3 listener
-    # and all the reading ends of pipes to active children
+    # Need to close all the duplicated file handles, this include the
+    # POP3 listener and all the reading ends of pipes to active
+    # children
 
     close $self->{server__} if ( defined( $self->{server__} ) );
 }
@@ -192,11 +196,13 @@ sub stop
 #
 # service
 #
-# service() is a called periodically to give the module a chance to do housekeeping work.
+# service() is a called periodically to give the module a chance to do
+# housekeeping work.
 #
-# If any problem occurs that requires POPFile to shutdown service() should return 0 and
-# the top level process will gracefully terminate POPFile including calling all stop()
-# methods.  In normal operation return 1.
+# If any problem occurs that requires POPFile to shutdown service()
+# should return 0 and the top level process will gracefully terminate
+# POPFile including calling all stop() methods.  In normal operation
+# return 1.
 #
 # ----------------------------------------------------------------------------
 sub service
@@ -211,15 +217,15 @@ sub service
     # of handles with data to read, if the handle is the server then
     # we're off.
 
-    if ( ( defined( $self->{selector__}->can_read(0) ) ) &&
-         ( $self->{alive_} ) ) {
+    if ( ( defined( $self->{selector__}->can_read(0) ) ) && # PROFILE BLOCK START
+         ( $self->{alive_} ) ) {                            # PROFILE BLOCK STOP
         if ( my $client = $self->{server__}->accept() ) {
 
             # Check to see if we have obtained a session key yet
 
             if ( $self->{api_session__} eq '' ) {
-                $self->{api_session__} =
-                    $self->{classifier__}->get_session_key( 'admin', '' );
+                $self->{api_session__} =                                   # PROFILE BLOCK START
+                    $self->{classifier__}->get_session_key( 'admin', '' ); # PROFILE BLOCK STOP
             }
 
             # Check that this is a connection from the local machine,
@@ -227,11 +233,11 @@ sub service
             # further processing.  We don't want to act as a proxy for
             # just anyone's email
 
-            my ( $remote_port, $remote_host ) = sockaddr_in(
-                                                    $client->peername() );
+            my ( $remote_port, $remote_host ) = sockaddr_in(                # PROFILE BLOCK START
+                                                    $client->peername() );  # PROFILE BLOCK STOP
 
-            if  ( ( ( $self->config_( 'local' ) || 0 ) == 0 ) ||
-                    ( $remote_host eq inet_aton( "127.0.0.1" ) ) ) {
+            if  ( ( ( $self->config_( 'local' ) || 0 ) == 0 ) ||        # PROFILE BLOCK START
+                    ( $remote_host eq inet_aton( "127.0.0.1" ) ) ) {    # PROFILE BLOCK STOP
 
                 # If we have force_fork turned on then we will do a
                 # fork, otherwise we will handle this inline, in the
@@ -247,9 +253,11 @@ sub service
                     # then process this request
 
                     if ( !defined( $pid ) || ( $pid == 0 ) ) {
-                        $self->{child_}( $self, $client,
-                            $self->{api_session__} );
-                        exit(0) if ( defined( $pid ) );
+                        $self->{child_}( $self, $client,        # PROFILE BLOCK START
+                            $self->{api_session__} );           # PROFILE BLOCK STOP
+                        if ( defined( $pid ) ) {
+                            &{$self->{childexit_}}( 0 );
+                        }
                     }
                 } else {
                     pipe my $reader, my $writer;
@@ -270,8 +278,9 @@ sub service
 #
 # forked
 #
-# This is called when some module forks POPFile and is within the context of the child
-# process so that this module can close any duplicated file handles that are not needed.
+# This is called when some module forks POPFile and is within the
+# context of the child process so that this module can close any
+# duplicated file handles that are not needed.
 #
 # There is no return value from this method
 #
@@ -393,10 +402,18 @@ sub get_response_
 
     # Retrieve a single string containing the response
 
-    my $selector = new IO::Select( $mail );
-    my ($ready) = $selector->can_read( (!$null_resp?$self->global_config_( 'timeout' ):.5) );
+    my $can_read = 0;
+    if ( $mail =~ /ssl/i ) {
+        $can_read = ( $mail->pending() > 0 );
+    }
+    if ( !$can_read ) {
+        my $selector = new IO::Select( $mail );
+        my ( $ready ) = $selector->can_read(                             # PROFILE BLOCK START
+            ( !$null_resp ? $self->global_config_( 'timeout' ) : .5 ) ); # PROFILE BLOCK STOP
+        $can_read = defined( $ready ) && ( $ready == $mail );
+    }
 
-    if ( ( defined( $ready ) ) && ( $ready == $mail ) ) {
+    if ( $can_read ) {
         $response = $self->slurp_( $mail );
 
         if ( $response ) {
@@ -466,10 +483,11 @@ sub echo_response_
 # $client      The handle to the mail client
 # $hostname    The host name of the remote server
 # $port        The port
-# $ssl         If set to 1 then the connection to the remote is established using SSL
+# $ssl         If set to 1 then the connection to the remote is established 
+#              using SSL
 #
-# Check that we are connected to $hostname on port $port putting the open handle in $mail.
-# Any messages need to be sent to $client
+# Check that we are connected to $hostname on port $port putting the
+# open handle in $mail.  Any messages need to be sent to $client
 #
 # ----------------------------------------------------------------------------
 sub verify_connected_
@@ -486,31 +504,116 @@ sub verify_connected_
 
     if ( $self->config_( 'socks_server' ) ne '' ) {
         require IO::Socket::Socks;
+        $self->log_( 0, "Attempting to connect to socks server at " # PROFILE BLOCK START
+                    . $self->config_( 'socks_server' ) . ":"
+                    . ProxyPort => $self->config_( 'socks_port' ) ); # PROFILE BLOCK STOP
+
         $mail = IO::Socket::Socks->new( # PROFILE BLOCK START
                     ProxyAddr => $self->config_( 'socks_server' ),
                     ProxyPort => $self->config_( 'socks_port' ),
                     ConnectAddr  => $hostname,
                     ConnectPort  => $port ); # PROFILE BLOCK STOP
-        $self->log_( 0, "Attempting to connect to socks server at "
-                    . $self->config_( 'socks_server' ) . ":"
-                    . ProxyPort => $self->config_( 'socks_port' ) );
     } else {
         if ( $ssl ) {
-            require IO::Socket::SSL;
-            $mail = IO::Socket::SSL->new( # PROFILE BLOCK START
-                        Proto    => "tcp",
-                        PeerAddr => $hostname,
-                        PeerPort => $port ); # PROFILE BLOCK STOP
-            $self->log_( 0, "Attempting to connect to SSL server at "
-                        . "$hostname:$port" );
+            eval {
+                require IO::Socket::SSL;
+            };
+            if ( $@ ) {
+                # Cannot load IO::Socket::SSL
+
+                $self->tee_( $client, "$self->{ssl_not_supported_error_}$eol" );
+                return undef;
+            }
+
+            $self->log_( 0, "Attempting to connect to SSL server at " # PROFILE BLOCK START
+                        . "$hostname:$port" );                        # PROFILE BLOCK STOP
+
+            if ( $^O eq 'MSWin32' ) { # PROFILE PLATFORM START MSWin32
+
+                # Workaround for avoiding intermittent password problem when
+                # using SSL. The problem occurs because IO::Socket->blocking
+                # is not supported on Windows.
+                # IO::Socket 1.30_01 which is included in Perl 5.10 seems to
+                # support blocking() on Windows. So we may remove this
+                # workaround when we move to Perl 5.10.
+
+                my $timeout = $self->global_config_( 'timeout' );
+                my $tt = time + $timeout;
+
+                $mail = IO::Socket::INET->new( # PROFILE BLOCK START
+                            Proto    => "tcp",
+                            PeerAddr => $hostname,
+                            PeerPort => $port,
+                            Timeout  => $timeout,
+                ); # PROFILE BLOCK STOP
+
+                if ( $mail ) {
+                    # Change the socket to non-blocking mode
+
+                    my $non_blocking = 1;
+                    ioctl( $mail, 0x8004667e, pack( 'L!', $non_blocking ) );
+
+                    $self->log_( 2, "Trying to upgrade socket $mail to SSL" );
+
+                    while ( $tt > time ) {
+                        # Upgrade the socket to SSL
+
+                        IO::Socket::SSL->start_SSL( # PROFILE BLOCK START
+                                $mail,
+                                Timeout  => $timeout,
+                        ); # PROFILE BLOCK STOP
+
+                        my $err = IO::Socket::SSL->errstr;
+                        last if ( $err eq '' );
+
+                        $self->log_( 1, "Got an error $err from start_SSL" );
+
+                        last if ( !defined $mail );
+
+                        my $vec = '';
+                        vec( $vec, $mail->fileno, 1 ) = 1;
+                        my $rv =  # PROFILE BLOCK START
+                            ( $err eq IO::Socket::SSL->SSL_WANT_READ )  ? select( $vec, undef, undef, $timeout ) :
+                            ( $err eq IO::Socket::SSL->SSL_WANT_WRITE ) ? select( undef, $vec, undef, $timeout ) :
+                            undef; # PROFILE BLOCK STOP
+
+                        last if ( !$rv );
+                    }
+
+                    if ( !defined $mail || ( ref $mail ne 'IO::Socket::SSL' ) ) {
+                        $self->log_( 0, "Failed to upgrade the socket to SSL" );
+                        $mail->close if defined $mail;
+                        undef $mail;
+                    } else {
+                        $self->log_( 2, "The socket $mail has successfully been upgraded" );
+
+                        # Restore to blocking mode
+
+                        $non_blocking = 0;
+                        ioctl( $mail, 0x8004667e, pack( 'L!', $non_blocking ) );
+                    }
+                }
+            } # PROFILE PLATFORM BLOCK STOP MSWin32
+            else {
+                $mail = IO::Socket::SSL->new( # PROFILE BLOCK START
+                            Proto    => "tcp",
+                            PeerAddr => $hostname,
+                            PeerPort => $port,
+                            Timeout  => $self->global_config_( 'timeout' ),
+                            Domain   => AF_INET,
+                ); # PROFILE BLOCK STOP
+            }
 
         } else {
+            $self->log_( 0, "Attempting to connect to POP server at " # PROFILE BLOCK START
+                        . "$hostname:$port" ); # PROFILE BLOCK STOP
+
             $mail = IO::Socket::INET->new( # PROFILE BLOCK START
                         Proto    => "tcp",
                         PeerAddr => $hostname,
-                        PeerPort => $port ); # PROFILE BLOCK STOP
-            $self->log_( 0, "Attempting to connect to POP server at "
-                        . "$hostname:$port" );
+                        PeerPort => $port,
+                        Timeout  => $self->global_config_( 'timeout' ),
+            ); # PROFILE BLOCK STOP
         }
     }
 
@@ -527,11 +630,13 @@ sub verify_connected_
                 binmode( $mail );
             }
 
-            # Wait 10 seconds for a response from the remote server and if
-            # there isn't one then give up trying to connect
+            if ( !$ssl || ( $mail->pending() == 0 ) ) {
+                # Wait 'timeout' seconds for a response from the remote server and
+                # if there isn't one then give up trying to connect
 
-            my $selector = new IO::Select( $mail );
-            last unless () = $selector->can_read($self->global_config_( 'timeout' ));
+                my $selector = new IO::Select( $mail );
+                last unless $selector->can_read($self->global_config_( 'timeout' ));
+            }
 
             # Read the response from the real server and say OK
 
@@ -543,6 +648,10 @@ sub verify_connected_
                 my $hit_newline = 0;
                 my $temp_buf;
 
+                # If we are on Windows, we will have to wait ourselves as
+                # we are not going to call IO::Select::can_read.
+                my $wait = ( ($^O eq 'MSWin32') && !($mail =~ /socket/i) ) ? 1 : 0;
+
                 # Read until timeout or a newline (newline _should_ be immediate)
 
                 for my $i ( 0..($self->global_config_( 'timeout' ) * 100) ) {
@@ -550,7 +659,11 @@ sub verify_connected_
                         $temp_buf = $self->flush_extra_( $mail, $client, 1 );
                         $hit_newline = ( $temp_buf =~ /[\r\n]/ );
                         $buf .= $temp_buf;
-                    } else {
+                        if ( $wait && ! length $temp_buf ) {
+                            select undef, undef, undef, 0.01;
+                        }
+                    }
+                    else {
                         last;
                     }
                 }
@@ -558,6 +671,13 @@ sub verify_connected_
 
             $self->log_( 1, "Connection returned: $buf" );
 
+            # If we cannot read any response from server, close the connection
+
+            if ( $buf eq '' ) {
+                close $mail;
+                last;
+            }
+
             $self->{connect_banner__} = $buf;
 
             # Clean up junk following a newline
@@ -570,6 +690,8 @@ sub verify_connected_
         }
     }
 
+    $self->log_( 0, "IO::Socket::INET or IO::Socket::SSL gets an error: $!" );
+
     # Tell the client we failed
     $self->tee_(  $client, "$self->{connection_failed_error_} $hostname:$port$eol" );
 
@@ -580,7 +702,8 @@ sub verify_connected_
 #
 # configure_item
 #
-#    $name            The name of the item being configured, was passed in by the call
+#    $name            The name of the item being configured, was passed in by
+#                     the call
 #                     to register_configuration_item
 #    $templ           The loaded template
 #
