@@ -2,9 +2,9 @@
 #
 # Services::IMAP::Client--- Helper module for the POPFile IMAP module
 #
-# Copyright (c) 2001-2007 John Graham-Cumming
+# Copyright (c) 2001-2009 John Graham-Cumming
 #
-#   $Revision: 1.1.2.5 $
+#   $Revision: 3407 $
 #
 #   This file is part of POPFile
 #
@@ -138,7 +138,9 @@ sub connect {
                                 PeerAddr => $hostname,
                                 PeerPort => $port,
                                 Timeout  => $timeout,
-                    );
+                                Domain   => AF_INET,
+                    )
+                    or $self->log_(0, "IO::Socket::SSL error: $@");
         }
         else {
             $imap = IO::Socket::INET->new(
@@ -146,7 +148,8 @@ sub connect {
                                 PeerAddr => $hostname,
                                 PeerPort => $port,
                                 Timeout  => $timeout,
-                    );
+                    )
+                    or $self->log_(0, "IO::Socket::INET error: $@");
         }
 
 
@@ -233,7 +236,7 @@ sub noop {
 #          information.
 #
 # IN:  $folder - name of the mailbox to be STATUSed
-# OUT: hasref with the keys UIDNEXT and UIDVALIDITY
+# OUT: hashref with the keys UIDNEXT and UIDVALIDITY
 # ----------------------------------------------------------------------------
 
 sub status {
@@ -947,8 +950,11 @@ sub uid_next {
 # ----------------------------------------------------------------------------
 sub check_uidvalidity {
     my $self    = shift;
-    my $folder  = shift or confess "gimme a folder";
-    my $new_val = shift or confess "gimme a new_val";
+    my $folder  = shift;
+    my $new_val = shift;
+
+    confess "check_uidvalidity needs a new uidvalidity!" unless defined $new_val;
+    confess "check_uidvalidity needs a folder name!" unless defined $folder;
 
     # Save old UIDVALIDITY value (if we have one)
     my $old_val = $self->uid_validity( $folder );
