@@ -8,7 +8,7 @@
 # header X-Text-Classification: into the header to tell the client
 # which category the message belongs in and much more...
 #
-# Copyright (c) 2001-2006 John Graham-Cumming
+# Copyright (c) 2001-2009 John Graham-Cumming
 #
 #   This file is part of POPFile
 #
@@ -37,7 +37,7 @@ $packing_list =~ s/[\\\/]$//;
 $packing_list .= '/popfile.pck';
 
 my $fatal = 0;
-my $log = '';
+my @log;
 
 if ( open PACKING, "<$packing_list" ) {
     while (<PACKING>) {
@@ -59,19 +59,19 @@ if ( open PACKING, "<$packing_list" ) {
                     $fatal = 1;
                     print STDERR "ERROR: POPFile needs Perl module $module, please install it.\n";
                 } else {
-                    $log .= "WARNING: POPFile may require Perl module $module; it is needed for \"$why\".\n";
+                    push @log, ("Warning: POPFile may require Perl module $module; it is needed only for \"$why\".");
                 }
             }
         }
     }
     close PACKING;
 } else {
-    $log .= "WARNING: Couldn't open POPFile packing list ($packing_list) so cannot check configuration\n";
+    push @log, ("Warning: Couldn't open POPFile packing list ($packing_list) so cannot check configuration (this probably doesn't matter)");
 }
 
 use strict;
 use locale;
-use lib defined($ENV{POPFILE_ROOT})?$ENV{POPFILE_ROOT}:'./';
+use lib defined( $ENV{POPFILE_ROOT} ) ? $ENV{POPFILE_ROOT} : '.';
 use POPFile::Loader;
 
 # POPFile is actually loaded by the POPFile::Loader object which does all
@@ -104,10 +104,14 @@ if ( $POPFile->CORE_config() ) {
     # If there were any log messages from the packing list check then
     # log them now
 
-    if ( $log ne '' ) {
-        $POPFile->get_module( 'POPFile::Logger' )->debug( 0, $log );
+    if ( $#log != -1 ) {
+        foreach my $m (@log) {
+            $POPFile->get_module( 'POPFile::Logger' )->debug( 0, $m );
+        }
     }
 
+    $POPFile->get_module( 'POPFile::Logger' )->debug( 0, "POPFile successfully started" );
+
     # This is the main POPFile loop that services requests, it will
     # exit only when we need to exit
 
